package pro.litvinovg.w2phtml.gui;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.text.AbstractDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.SimpleAttributeSet;

import ch.qos.logback.classic.Level;
import ch.qos.logback.classic.PatternLayout;
import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.core.AppenderBase;

public class LogAppender extends AppenderBase<ch.qos.logback.classic.spi.ILoggingEvent> {

  private static SimpleAttributeSet ERROR_ATT, WARN_ATT, INFO_ATT, DEBUG_ATT, TRACE_ATT, RESTO_ATT;
  private PatternLayout patternLayout;
  
	@Override
   protected void append(ch.qos.logback.classic.spi.ILoggingEvent event) {
       // Formata mensagem do log
       String formattedMsg = patternLayout.doLayout( event);

       // Forma segura de atualizar o JTextpane
       SwingUtilities.invokeLater(() -> {
           // Alias para o JTextPane no frame da aplicação
           JTextPane textPane = null;//App.MAIN_FORM.getTextPane();

           try {
               // Trunca linhas para economizar memória
               // Quando atingir 2000 linhas, eu quero que
               // apague as 500 primeiras linhas
               int limite = 1000;
               int apaga = 200;
               if (textPane.getDocument().getDefaultRootElement().getElementCount() > limite) {
                   int end = getLineEndOffset(textPane, apaga);
                   replaceRange(textPane, null, 0, end);
               }

               // Decide qual atributo (estilo) devo usar de acordo com o nível o log
               if (event.getLevel() == Level.ERROR)
                   textPane.getDocument().insertString(textPane.getDocument().getLength(), formattedMsg, ERROR_ATT);
               else if (event.getLevel() == Level.WARN)
                   textPane.getDocument().insertString(textPane.getDocument().getLength(), formattedMsg, WARN_ATT);
               else if (event.getLevel() == Level.INFO)
                   textPane.getDocument().insertString(textPane.getDocument().getLength(), formattedMsg, INFO_ATT);
               else if (event.getLevel() == Level.DEBUG)
                   textPane.getDocument().insertString(textPane.getDocument().getLength(), formattedMsg, DEBUG_ATT);
               else if (event.getLevel() == Level.TRACE)
                   textPane.getDocument().insertString(textPane.getDocument().getLength(), formattedMsg, TRACE_ATT);
               else
                   textPane.getDocument().insertString(textPane.getDocument().getLength(), formattedMsg, RESTO_ATT);

           } catch (BadLocationException e) {
               // Faz nada
           }

           // Vai para a última linha
           textPane.setCaretPosition(textPane.getDocument().getLength());
       });
   }

	 public void start() {
		 patternLayout = new PatternLayout();
		 patternLayout.setContext(getContext());
		 patternLayout.setPattern("%d{HH:mm:ss.SSS} [%thread] %-5level %logger{36} - %msg%n");
		 patternLayout.start();

		 super.start();
	 }

	 private int getLineCount(JTextPane textPane) {
		 return textPane.getDocument().getDefaultRootElement().getElementCount();
	 }


  private void replaceRange(JTextPane textPane, String str, int start, int end) throws IllegalArgumentException {
    if (end < start) {
        throw new IllegalArgumentException("end before start");
    }
    Document doc = textPane.getDocument();
    if (doc != null) {
        try {
            if (doc instanceof AbstractDocument) {
                ((AbstractDocument)doc).replace(start, end - start, str, null);
            }
            else {
                doc.remove(start, end - start);
                doc.insertString(start, str, null);
            }
        } catch (BadLocationException e) {
            throw new IllegalArgumentException(e.getMessage());
        }
    }
}
  private int getLineEndOffset(JTextPane textPane, int line) throws BadLocationException {
    int lineCount = getLineCount(textPane);
    if (line < 0) {
        throw new BadLocationException("Negative line", -1);
    } else if (line >= lineCount) {
        throw new BadLocationException("No such line", textPane.getDocument().getLength()+1);
    } else {
        Element map = textPane.getDocument().getDefaultRootElement();
        Element lineElem = map.getElement(line);
        int endOffset = lineElem.getEndOffset();
        // hide the implicit break at the end of the document
        return ((line == lineCount - 1) ? (endOffset - 1) : endOffset);
    }
}
}
